<?php

namespace Drupal\autoslave\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Component\Utility\Crypt;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Exception;
use Drupal\Core\Database\Database;
use mysqli;
use Symfony\Component\HttpFoundation\JsonResponse;

class AutoslaveDashboardController extends ControllerBase {

  /**
   * Page callback, make this ajax available to provide a link on the autoslave_dashboard_page
   * to trigger autoslave_dashboard_cron path is: admin/config/system/autoslave/cron.
   */
  public function autoslaveCron() {
    $this->__autoslave_dashboard_cron();
    sleep(1);// Nice wait animation on interface.
    return new JsonResponse(array('status' => TRUE, 'result' => 'ok'));
  }

  /**
   * Implements the dashboard page rendering.
   */
  public function autoslaveDashboard() {

    if (!autoslave_is_driver_loaded()) {
			\Drupal::messenger()->addMessage('AutoSlave driver is not loaded', 'warning');
    }

    $asd_formatted_hashlink = NULL; // Pass $asd_full_link to the theme autoslave_dashboard for autoslave_dashboard.tpl.php.
    $master_host_port = array();//Do master processing only once.
    if (\Drupal::currentUser()->hasPermission('administer site configuration')) {
      $asd_hashkey = $this->_autoslave_dashboard_get_hash_from_hostname();
      $asd_link = (($_SERVER['HTTPS'] == 'off')? 'http://' : 'https://') . $_SERVER['SERVER_NAME'] .
        '/autoslave/dashboard?' .  \Drupal::state()->get('autoslave_dashboard_url_key', 'hash') . '=' . $asd_hashkey;
      $asd_link_text = t("You can use ") .$asd_link . t(" to view this dashboard without being logged in.");
      $options['attributes']['title'] = $asd_link_text;
      $options['query'][\Drupal::state()->get('autoslave_dashboard_url_key', 'hash')] = $asd_hashkey;
      $asd_formatted_hashlink = Link::fromTextAndUrl($asd_link_text, Url::fromRoute('autoslave.dashboard', $options))->toString();
    }


    //do processing for json, output as a string, pass it to the theme variables.
    $array_of_servers = array();
    $array_for_json = array(); // To be used for the autoslave_server_stats array processing.
    $array_slave_status = array(); // To be used for the autoslave_slave_status array processing.
    $result_num_servers = \Drupal::database()->select('autoslave_server_stats', 'a')
      ->fields('a', array('server_num'))
      ->distinct()
      ->execute()
      ->fetchAll();
    $num_servers = count($result_num_servers);
    foreach ($result_num_servers as $srvnum_server) {
      //Start the autoslave_server_stats json_data array processing.
      $srvnum = $srvnum_server->server_num;
      $array_of_servers[] = $srvnum;
      $data = array();
      $number_of_deltas = (int) (\Drupal::state()->get('autoslave_dashboard_deltas', 30) + 1);
      if (!is_numeric($number_of_deltas)) {
        $number_of_deltas = 31; // Number of bars = $number_of_deltas -1.
      }
      $countResults = \Drupal::database()->select('autoslave_server_stats', 'a')
        ->fields('a', array('server_num', 'n_select', 'n_insert', 'n_update', 'ts'))
        ->condition('server_num', $srvnum,'=')
        ->execute()
        ->fetchAll();
      $total_number_of_stats = count($countResults);
      $start_range = $total_number_of_stats-$number_of_deltas;
      if ($start_range < 0) {
        $start_range = 0;
      }
      $results = \Drupal::database()->select('autoslave_server_stats', 'a')
        ->fields('a', array('server_num', 'n_select', 'n_insert', 'n_update', 'ts'))
        ->condition('server_num', $srvnum,'=')
        ->range($start_range, $total_number_of_stats)
        ->orderBy('ts', 'ASC')
        ->execute()
        ->fetchAll();
      $prev_select = null;
      $prev_insert = null;
      $prev_update = null;
      foreach ($results as $row) {
        if ($prev_select === null) {
          $prev_select = $row->n_select;
          $prev_insert = $row->n_insert;
          $prev_update = $row->n_update;
          continue;
        }
        $array_for_json[$srvnum][] = array($row->n_select - $prev_select, $row->n_insert - $prev_insert, $row->n_update - $prev_update);
        $prev_select = $row->n_select;
        $prev_insert = $row->n_insert;
        $prev_update = $row->n_update;
      }
      //End of the autoslave_server_stats json_data array processing.
      //Start the autoslave_slave_status status_data array.
      $results = \Drupal::database()->select('autoslave_slave_status', 'a')
        ->fields('a', array('server_num',
          'io_state',
          'master_host',
          'master_port',
          'master_uuid',
          'auto_position',
          'io_running',
          'sql_running',
          'last_error',
          'last_io_error',
          'last_sql_error',
          'sql_running_state',
          'ts'))
        ->condition('server_num', $srvnum,'=')
        ->execute()
        ->fetchAll();
      foreach ($results as $row) {
        $server_status = sprintf("<table class='db-charts-table'><thead><tr><th>Server Number : " .
          " %d</th><th>Master is : %s <span class='db-charts-masteronport'>on port</span> %s</th></tr></thead>" .
          "<tbody>" .
          "<tr><td><span class='label'>IO Running :</span> %s</td><td><span class='label'>SQL Running :</span> %s</td></tr>" .
          "<tr><td><span class='label'>Master_UUID :</span> %s</td><td><span class='label'>Auto_Position :</span> %s</td></tr>" .
          "<tr><td colspan='2'><span class='label'>Last Error :</span> %s</td></tr>" .
          "<tr><td colspan='2'><span class='label'>Last IO Error :</span> %s</td></tr>" .
          "<tr><td colspan='2'><span class='label'>Last SQL Error :</span> %s</td></tr>" .
          "<tr><td colspan='2'><span class='label'>SQL Running State :</span> %s</td></tr>" .
          "<tr><td colspan='2'><span class='label'>Last Updated :</span> %s</td></tr></tbody></table>",
          $srvnum,
          $row->master_host,
          $row->master_port,
          $row->io_running,
          $row->sql_running,
          $row->master_uuid,
          $row->auto_position,
          $row->last_error,
          $row->last_io_error,
          $row->last_sql_error,
          $row->sql_running_state,
          $row->ts
        );
        $array_slave_status[$srvnum] = $server_status;
      }
      //End the autoslave_slave_status status_data array.
    }

    $array_of_masters = array();
    if (!autoslave_is_driver_loaded()) {
      //AUTOSLAVE DRIVER IS NOT LOADED.
      $array_for_json = NULL;
      $msg = t('autoslave driver is not loaded');
      $not_available = sprintf("<table class='db-charts-table'><thead><tr><th> %s" .
        "</th></tr></thead><tbody><tr><td>%s</td></tr></tbody></table>", $msg, $msg, $msg);
      $array_slave_status = array();
      $array_slave_status[1] = $not_available;
      $array_master_status = array();
      $array_master_status[1] = '';//$not_available;
      $array_of_servers = array();
      $array_of_servers[] = 1;
      $num_servers = 1;
      $asd_formatted_hashlink = NULL;
    }
    else {
      $array_master_status = $this->__autoslave_master_status();
      $num_of_masters = count($array_master_status);
      for ($m = 1; $m <= $num_of_masters; $m++) {
        $array_of_masters[] = $m;
      }
    }
    if (\Drupal::moduleHandler()->moduleExists('elysia_cron')) {
      if (elysia_cron_is_job_disabled('autoslave_cron')) {
				\Drupal::messenger()->addMessage("The elysia_cron module is enabled, the 'autoslave_cron' job is 'PAUSED'. " .
          "This is to prevent autoslave from overriding your manual invalidation because normally autoslave_cron " .
          "will test your server connections and automatically bring them back into load.  Normally this is what " .
          "you want however in a manual override you are probably doing maintenance on your server and don't" .
          " want it to be hit.  The 'autoslave_cron' job will be UN-PAUSED when you resume normal operation." .
          "If you are planning server maintenance on the invalidated server now would be a good time to run" .
          " 'stop slave' on that host.",
          'warning', TRUE);
      }
    }


    $dashboard = [
      'json_data' => json_encode($array_for_json, JSON_PRETTY_PRINT),
      'json_slave_status' => json_encode($array_slave_status, JSON_PRETTY_PRINT),
      'json_master_status' => json_encode($array_master_status, JSON_PRETTY_PRINT),
      'json_servers' => json_encode($array_of_servers, JSON_PRETTY_PRINT),
      'json_masters' => json_encode($array_of_masters, JSON_PRETTY_PRINT),
      'num_servers' => $num_servers,
      'num_masters' => count($array_master_status),
      'hashkey_link' => $asd_formatted_hashlink,
      'module_path' => drupal_get_path('module', 'autoslave')
      // Variables go here, you can add more variables like json_data.
      // These variables are used in templates/autoslave_dashboard.tpl.php.
    ];

    $build = [
      '#theme' => 'autoslave_dashboard',
      '#dashboard' => $dashboard,
      '#attached' => [
        'library' => [
          'autoslave/autoslave',
        ],
      ],
    ];

    return $build;
  }

  /**
   * Generate hash from node ID.
   *
   * @param int $nid
   *   Node ID, nid.
   *
   * @return string
   *   Base-64 encoded, URL-safe sha-256 hash based on node ID.
   */
  public function _autoslave_dashboard_get_hash_from_hostname() {
    $hostname = $_SERVER['SERVER_NAME'];
    return Crypt::hashBase64(\Drupal::service('private_key')->get() . $hostname);
  }

  public function __autoslave_master_status() {

    $databases = Database::getAllConnectionInfo();
    $output = '';
    $master_num_serv = 0;
    $some_invalidation_is_kicked_in = FALSE;
    foreach ($databases as $key => $targets) {
      foreach ($targets as $target => $conninfo) {
        $conninfos = empty($conninfo['driver']) ? $conninfo : array($conninfo);
        foreach ($conninfos as $conninfo) {
          if ($conninfo['driver'] != 'autoslave') {
            continue;
          }
          $output .= "<h2>[$key][$target]</h2>";
          try {
            $conn = Database::getConnection($target, $key);
            $pool = $conn->getPool();
            $invalidation_file = '';
            $invalidation_message = t('Autoslave is in default operation mode');
            if (isset($conn->connectionOptions['invalidation path'])) {
              $invalidation_file = $conn->connectionOptions['invalidation path'] . "/autoslave-invalidation-$key.inc";
              if (file_exists($invalidation_file)) {
                $some_invalidation_is_kicked_in = TRUE;
                $invalidation_message = t('Invalidation mode enabled with @filepath', array('@filepath' => $invalidation_file));
              }
            }
            else {
              $invalidation_file = "public://autoslave-invalidation-$key.inc";
              if ($wrapper = \Drupal::service('stream_wrapper_manager')->getViaUri($invalidation_file)) {
                $invalidation_file_real = $wrapper->realpath();
              }
              if (file_exists($invalidation_file_real)) {
                $some_invalidation_is_kicked_in = TRUE;
                $invalidation_message = t('Invalidation mode enabled with @filepath', array('@filepath' => $invalidation_file_real));
              }
            }
            foreach ($pool['master'] as $target_master => $conninfos) {
              if ($conn->determineMasterTarget() == $target_master) {
                $target_master_info = '<strong>' . $target_master . '</strong>';
                $master_num_serv++;
                //watchdog('autoslave_DEBUG_MASTER','<pre>@conninfos</pre>', array('@conninfos'=> print_r($conninfos, TRUE)));
                $master_host = isset($conninfos[$master_num_serv]['host']) ? $conninfos[$master_num_serv]['host'] : t('Error reading hostname');
                $master_port = isset($conninfos[$master_num_serv]['port']) ? $conninfos[$master_num_serv]['port'] : 3306;
                $server_status = sprintf("<table data-autoslave-invalidation='%d' data-target-master='%s' data-target-host='%s' data-target-port='%s' class='db-charts-table is-master'><thead><tr><th>%s at host %s port %s" .
                  "<span class='db-charts-masteronport'></span></th></tr></thead>" .
                  "<tbody><tr><td><span class='label'><strong>Currently live as a master (autoslave valid).</strong></span></td></tr><tr><td><span class='label'>" .
                  "Key: %s Target: %s <br>Pool: Master</span></td></tr><tr><td><span class='label'>%s</span></td></tr>",
                  $some_invalidation_is_kicked_in, $target_master, $master_host, $master_port, $target_master, $master_host, $master_port, $key, $target, $invalidation_message
                );
              }
              else if ($conn->determineMasterTarget() != $target_master) {
                $target_master_info = '<strong>' . $target_master . '</strong>';
                $master_num_serv++;
                $autoslave_status_for_server = 0;
                $server_status_message = t('currently disabled (autoslave invalid).');
                if (isset($conninfos[$master_num_serv]['status'])) {
                  $autoslave_status_for_server = $conninfos[$master_num_serv]['status'];
                }
                if ($autoslave_status_for_server || !$some_invalidation_is_kicked_in) {
                  $server_status_message = t('currently enabled (autoslave valid) but not live as a master.');
                }
                else {
                  $autoslave_status_for_server = 0;
                }
                //watchdog('autoslave_DEBUG_TEMPORARY','<pre>@conninfos</pre>', array('@conninfos'=> print_r($conninfos, TRUE)));
                $master_host = isset($conninfos[$master_num_serv]['host']) ? $conninfos[$master_num_serv]['host'] : t('Error reading hostname');
                $master_port = isset($conninfos[$master_num_serv]['port']) ? $conninfos[$master_num_serv]['port'] : 3306;
                $server_status = sprintf("<table data-autoslave-invalidation='%d' data-target-master='%s' data-target-host='%s' data-target-port='%s' class='db-charts-table is-slave'><thead><tr><th>%s at host %s port %s" .
                  "<span class='db-charts-masteronport'></span></th></tr></thead>" .
                  "<tbody><tr><td><span class='label'><strong>Server is %s</strong></span></td></tr><tr><td><span class='label'>" .
                  "Key: %s Target: %s <br>Pool: Master</span></td></tr><tr><td><span class='label'>%s</span></td></tr>",
                  $some_invalidation_is_kicked_in, $target_master, $master_host, $master_port, $target_master, $master_host, $master_port, $server_status_message, $key, $target, $invalidation_message
                );
              }
              $array_master_status[$master_num_serv] = $server_status;
            }
          }
          catch (Exception $e) {
						\Drupal::messenger()->addMessage('Error connecting to: [@key][@target]', array(
              '@key' => $key,
              '@target' => $target
            ), 'error');
          }
        }
      }
    }

    return $array_master_status;
  }


  /**
   * Local function, see autoslave_cron collect statistics from the mysql;
   */
  public static function __autoslave_dashboard_cron() {
    $databases = Database::getAllConnectionInfo();
    $debug_mode = \Drupal::state()->get('autoslave_debug_mode');
    $n_select = 0;
    $n_insert = 0;
    $n_update = 0;
    $srv_num = 0;
    $host_port = array();
    foreach ($databases as $key => $targets) {
      foreach ($targets as $target => $conninfo) {
        $conninfos = empty($conninfo['driver']) ? $conninfo : array($conninfo);
        foreach ($conninfos as $conninfo) {
          if ($conninfo['driver'] != 'mysql') {
            continue;// Skip non-mysql drivers.
          }
          $n_select = 0;
          $n_insert = 0;
          $n_update = 0;
          $conninfo_host = isset($conninfo['host'])? $conninfo['host'] : '';
          $conninfo_port = isset($conninfo['port'])? $conninfo['port'] : '3306';
          if (empty($conninfo_port)) {
            $conninfo_port = 3306;
          }
          if (empty($conninfo_host) || !isset($conninfo['username'])) {
            continue;
          }
          $hostport_string = $conninfo_port . $conninfo_host;
          if (in_array($hostport_string, $host_port)) {
            // This server host and port was previously processed, skipping.
            continue;
          }
          if ($debug_mode) {
            watchdog('autoslave_cron_debug', "[@key][@target] port = @port host = @host server_num = @srv_num", array(
              '@key' => $key,
              '@target' => $target,
              '@port' => $conninfo_port,
              '@host' => $conninfo_host,
              '@srv_num' => $srv_num,
            ), WATCHDOG_NOTICE);
          }
          $host_port[] =  $hostport_string;
          $test_for_slave = stripos($target, 'slave'); //Tests to see if this target is a 100% slave or not.
          $srv_num = (isset($conninfo['autoslave_id']) ? $conninfo['autoslave_id'] : $srv_num=$srv_num+1);
          $sdb = ''; // keep this in scope for this loop.
          // BEGIN PROCESSING FOR NEW autoslave_server_stats.
          try
          {
            /*
             * Intentionally changed this query to mysqli from Database::getConnection($target, $key)->query.
             * Made this change to ensure going to the intended database because
             * Database::getConnection for target slave occasionally would point to the master
             * grabbing incorrect statistics and skewing results.  mysqli connection ensures bypassing autoslave.
             **/
            $sdb = new mysqli($conninfo_host, $conninfo['username'], $conninfo['password'], '', $conninfo_port);
            $result = $sdb->query("show global status where Variable_name='Com_select'");
            if (empty($result)) {
              $n_select = 0;
              $n_update = 0;
              $n_insert = 0;
							\Drupal::messenger()->addMessage("Insufficient priviledges to run the mysql command \"show global status where Variable_name='Com_select';\"" .
                ' on server_num @srv_num [@key][@target] at host: @host on port @port.  You must grant the ' .
                'correct permissions to @user@\'@host\';"', array(
                '@host' => isset($conninfo['host']) ? $conninfo['host'] : 'host',
                '@user' => isset($conninfo['username']) ? $conninfo['username'] : 'username',
                '@key' => $key,
                '@port' => !empty($conninfo['port']) ? $conninfo['port'] : '3306',
                '@target' => $target,
                '@srv_num' =>  $srv_num,
              ), 'warning', TRUE);
            }
            else {
              $row = $result->fetch_array();
              $n_select = $row['Value'];
              $result = $sdb->query("show global status where Variable_name='Com_insert'");
              $row = $result->fetch_array();
              $n_insert = $row['Value'];
              $result = $sdb->query("show global status where Variable_name='Com_update'");
              $row = $result->fetch_array();
              $n_update = $row['Value'];
            }
          }
          catch (Exception $e)
          {
            // do some logging, error handling...
            $n_select = 0;
            watchdog('autoslave_cron', "Error connecting to: [@key][@target] port = @port host = @host", array(
              '@key' => $key,
              '@target' => $target,
              '@port' => $conninfo_port,
              '@host' => $conninfo_host,
            ), WATCHDOG_ERROR);
          }
          try {
            $insert_return_code = \Drupal::database()->insert('autoslave_server_stats') // Table name no longer needs {}
            ->fields(array(
              'server_num' => $srv_num,
              'n_select' => $n_select,
              'n_insert' => $n_insert,
              'n_update' => $n_update,
              'ts' => date('Y-m-d H:i:s'),
            ))
              ->execute();
          }
          catch (Exception $e) {
            watchdog('autoslave_cron', 'Error inserting into table autoslave_server_stats on: [@key][@target]', array(
              '@key' => $key,
              '@target' => $target
            ), WATCHDOG_ERROR);
          }
          // END PROCESSING FOR NEW autoslave_server_stats.
          // BEGIN TRIMMING OLDER autoslave_server_stats.
          try {
            // Keep the database trimmed.
            $number_of_days_to_keep = 4;
            $del_before = date('Y-m-d H:i:s', time() - $number_of_days_to_keep * 86400);
            // Drupal 7
            $num_deleted = \Drupal::database()->delete('autoslave_server_stats')
              ->condition('server_num', $srv_num)
              ->condition('ts', $del_before, '<')
              ->execute();
          }
          catch (Exception $e) {
            watchdog('autoslave_cron', 'Error deleting rows from the autoslave_server_stats table on: [@key][@target]', array(
              '@key' => $key,
              '@target' => $target
            ), WATCHDOG_ERROR);
          }
          // END TRIMMING OLDER autoslave_server_stats.
          // BEGIN autoslave_slave_status processing , note this will only get data from slaves.
          try {
						\Drupal::database()->delete('autoslave_slave_status')
              ->condition('server_num', $srv_num, '=')
              ->execute();
          }
          catch (Exception $e) {
            //Do nothing, in this case it's ok, we needed to delete the old $srv_num row, and if there wasn't one already it is ok.
            watchdog('autoslave_cron', 'Exception message: @e', array('@e' => $e), WATCHDOG_NOTICE);
          }
          try {
            // Update the autoslave_slave_status table.
            $fields_array = array();
            if ($test_for_slave !== FALSE) {
              sleep(2);// Sleep 2 seconds. (mostly because of my lame dev setup (3 mysql servers on same slow vm host different ports).
            }
            $bShowSlaveSuccess = FALSE;
            /*
             * Intentionally changed this query to mysqli from Database::getConnection($target, $key)->query.
             * Made this change to ensure going to the intended database because
             * Database::getConnection for target slave occasionally would point to the master
             * grabbing information from the incorrect server.  mysqli connection ensures bypassing autoslave.
             **/
            $sdb = new mysqli($conninfo_host, $conninfo['username'], $conninfo['password'], '', $conninfo_port);
            $result = $sdb->query("show slave status");
            if (empty($result)) {
              throw new Exception('Need permissions for replication client.');
            }
            $row = $result->fetch_array();
            if (!empty($row)) {
              $bShowSlaveSuccess = TRUE;
            }
            $fields_array = array(
              'server_num' => $srv_num,
              'io_state' => substr($row['Slave_IO_State'],0, 256),
              'master_host' => substr($row['Master_Host'],0, 256),
              'master_port' => $row['Master_Port'],
              'master_uuid' => substr($row['Master_UUID'],0, 256),
              'auto_position' => $row['Auto_Position'],
              'io_running' => $row['Slave_IO_Running'],
              'sql_running' => $row['Slave_SQL_Running'],
              'last_error' => $row['Last_Error'],
              'last_io_error' => $row['Last_IO_Error'],
              'last_sql_error' => $row['Last_SQL_Error'],
              'sql_running_state' => substr("host $conninfo_host port $conninfo_port " . $row['Slave_SQL_Running_State'],0 ,256),
              'ts' => date('Y-m-d H:i:s'),
            );
            if (!$bShowSlaveSuccess) {
              $text_master_or_slave = 'Master is up and running.';
              if ($test_for_slave !== FALSE) {
                $text_master_or_slave = 'Slave is up and running and currently processing the relay logs.';
              }
              $fields_array = array(
                'server_num' => $srv_num,
                'io_state' => '',
                'master_host' => isset($conninfo['host']) ? $conninfo['host'] : 'host',
                'master_port' => '',
                'master_uuid' => '',
                'io_running' => '',
                'sql_running' => '',
                'last_error' => '',
                'last_io_error' => '',
                'last_sql_error' => '',
                'sql_running_state' => "host $conninfo_host port $conninfo_port " . $text_master_or_slave,
                'ts' => date('Y-m-d H:i:s'));
              if ($test_for_slave !== FALSE) {
                watchdog('autoslave_cron_helper', 'No records found for show slave status on [@key][@target] at server_num @srv_num ' .
                  '@host on port @port appears to be a slave server.  Have you granted REPLICATION CLIENT to the mysql user @user ' .
                  ' to run "show slave status" on: [@key][@target]?', array(
                  '@host' => isset($conninfo['host']) ? $conninfo['host'] : 'host',
                  '@user' => isset($conninfo['username']) ? $conninfo['username'] : 'username',
                  '@key' => $key,
                  '@port' => !empty($conninfo['port']) ? $conninfo['port'] : '3306',
                  '@target' => $target,
                  '@srv_num' =>  $srv_num,
                ), WATCHDOG_NOTICE);
              }
            }
            $insert_return_code = \Drupal::database()->insert('autoslave_slave_status')->fields($fields_array)->execute();
          }
          catch (Exception $e) {
            if ($test_for_slave) {
              watchdog('autoslave_cron_helper', '(Slave server) Insufficient priviledges to run the mysql command "show slave status;"' .
                ' on server_num @srv_num [@key][@target] at host: @host on port @port.  You must grant the ' .
                'correct permissions as follows: "grant REPLICATION CLIENT on *.* to @user@\'@host\';" ' .
                'server_num ' . $srv_num . ' at host: @host on port @port appears to be a slave server. ' .
                ' The mysql user @user is probably lacking permissions to run "show slave status" on: ' .
                '[@key][@target].  Full exception message: @e', array(
                '@host' => isset($conninfo['host']) ? $conninfo['host'] : 'host',
                '@user' => isset($conninfo['username']) ? $conninfo['username'] : 'username',
                '@key' => $key,
                '@port' => !empty($conninfo['port']) ? $conninfo['port'] : '3306',
                '@target' => $target,
                '@srv_num' =>  $srv_num,
                '@e' => $e,
              ), WATCHDOG_NOTICE);
            }
            else {
              watchdog('autoslave_cron_helper', '(Master or secondary master. server) You must grant the correct permissions on server_num @srv_num as follows: ' .
                '"grant REPLICATION CLIENT on *.* to @user@\'@host\';"  the @user is probably lacking permissions for ' .
                '"show slave status" on: [@key][@target].  Full exception message: @e', array(
                '@host' => isset($conninfo['host']) ? $conninfo['host'] : 'host',
                '@user' => isset($conninfo['username']) ? $conninfo['username'] : 'username',
                '@key' => $key,
                '@port' => !empty($conninfo['port']) ? $conninfo['port'] : '3306',
                '@target' => $target,
                '@srv_num' =>  $srv_num,
                '@e' => $e,
              ), WATCHDOG_NOTICE);
            }
						\Drupal::messenger()->addMessage('Insufficient priviledges to run the mysql command "show slave status;"' .
              ' on server_num @srv_num [@key][@target] at host: @host on port @port.  You must grant the ' .
              'correct permissions as follows: "grant REPLICATION CLIENT on *.* to @user@\'@host\';"', array(
              '@host' => isset($conninfo['host']) ? $conninfo['host'] : 'host',
              '@user' => isset($conninfo['username']) ? $conninfo['username'] : 'username',
              '@key' => $key,
              '@port' => !empty($conninfo['port']) ? $conninfo['port'] : '3306',
              '@target' => $target,
              '@srv_num' =>  $srv_num,
            ), 'warning', TRUE);
          }
          // END autoslave_slave_status processing , note this will only get data from slaves.
          //grant REPLICATION CLIENT on *.* to user@'host';*/
        }
      }
    }
  }

  /**
   * Autoslave json callback for ajax manual override invalidate server admin/autoslave/serverhost/serverport/invalidate.
   * returns json.
   */
  public function autoslaveInvalidate($hostname,$port) {
    $server_host = $hostname;
    $server_port = $port;
    $invalidation_is_kicked_in = FALSE;
    // Make t() function calls prior to invalidation, this avoids a 500 error.
    // A 500 error avoid because this thread db connection will be invalidated
    // so avoid db access, do it now, avoid immediately after invalidation.
    $invalidation_failed = t('Autoslave is in default operation mode, invalidation did not kick in.');
    $invalidation_message = t('Autoslave is in invalidation mode using this file: ');
    $stop_slave_msg = t(" for host: @host on port @port.  If you're planning server maintenance on host " .
      "@host port @port down then you now should run 'stop slave'", array(
      '@host' => $server_host,
      '@port' => $server_port,
    ));
    $elysia_msg = '';

    if (empty($server_host)) {
			\Drupal::messenger()->addMessage($invalidation_failed . t(" for host: @host on port @port.", array(
          '@host' => $server_host,
          '@port' => $server_port,
        )), 'warning', TRUE);
      return new JsonResponse(array('status'=> FALSE, 'data' => $invalidation_failed), JSON_PRETTY_PRINT);
    }
    if (empty($server_port)) {
      $server_port = 3306; // Default the server port to 3306 when no server port is provided.
    }
    if (\Drupal::moduleHandler()->moduleExists('elysia_cron')) {
      elysia_cron_set_job_disabled('autoslave_cron', TRUE);
      $elysia_msg = t(" ALSO TEMPORARILY DISABLED ELYSIA CRON FOR autoslave ");
    }
    $databases = Database::getAllConnectionInfo();
    foreach ($databases as $key => $targets) {
      foreach ($targets as $target => $conninfo) {
        $conninfos = empty($conninfo['driver']) ? $conninfo : array($conninfo);
        foreach ($conninfos as $conninfo) {
          if ($conninfo['driver'] != 'autoslave') {
            continue;
          }
          try {
            $conn = Database::getConnection($target, $key);
            $pool = $conn->getPool();

            $invalidation_file = '';
            foreach ($pool['master'] as $target_master => $masterconninfos) {
              //if ($conn->determineMasterTarget() == $target_master) {
              // $conn for to the current live master.
              //}
              foreach($masterconninfos as $id => $connection) {
                if (isset($connection['port']) && isset($connection['host'])) {
                  if ($server_port == $connection['port'] && $server_host == $connection['host']) {
                    $conn->invalidateConnection($id);
                  }
                }
              }
              foreach ($pool['slave'] as $target_slave => $slaveconninfos) {
                foreach($slaveconninfos as $id => $connection) {
                  if (isset($connection['port']) && isset($connection['host'])) {
                    if ($server_port == $connection['port'] && $server_host == $connection['host']) {
                      $conn->invalidateConnection($id);
                    }
                  }
                }
              }
            }
            if (isset($conn->connectionOptions['invalidation path'])) {
              $invalidation_file = $conn->connectionOptions['invalidation path'] . "/autoslave-invalidation-$key.inc";
              if (file_exists($invalidation_file)) {
                $invalidation_is_kicked_in = TRUE;
                $invalidation_message.= $invalidation_file;
              }
            }
            else {
              $invalidation_file = "public://autoslave-invalidation-$key.inc";
              if ($wrapper = \Drupal::service('stream_wrapper_manager')->getViaUri($invalidation_file)) {
                $invalidation_file_real = $wrapper->realpath();
              }
              if (file_exists($invalidation_file_real)) {
                $invalidation_is_kicked_in = TRUE;
                $invalidation_message.= $invalidation_file;
              }
            }
          }
          catch(Exception $e) {
            watchdog('autoslave_invalidate_server','<pre>@e</pre>', array('@e'=> print_r($e,TRUE)));
          }
        }
      }
    }
    if ($invalidation_is_kicked_in) {
      return new JsonResponse(array('status'=> TRUE, 'data' => $invalidation_message . $elysia_msg .
        $stop_slave_msg));
    }
    else {
      if (\Drupal::moduleHandler()->moduleExists('elysia_cron')) {
        elysia_cron_set_job_disabled('autoslave_cron', FALSE);
      }
			\Drupal::messenger()->addMessage($invalidation_failed . t(" for host: @host on port @port.", array(
          '@host' => $server_host,
          '@port' => $server_port,
        )), 'error', TRUE);
      return new JsonResponse(array('status'=> FALSE, 'data' => $invalidation_failed));
    }
  }

}